package org.xbib.net.security.ssl.model;

import java.util.List;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLParameters;
import javax.net.ssl.X509ExtendedKeyManager;
import javax.net.ssl.X509ExtendedTrustManager;

/**
 * <strong>NOTE:</strong>
 * Please don't use this class directly as it is part of the internal API. Class name and methods can be changed any time.
 */
public final class SSLMaterial {

    private SSLContext sslContext;
    private X509ExtendedKeyManager keyManager;
    private X509ExtendedTrustManager trustManager;
    private HostnameVerifier hostnameVerifier;
    private SSLParameters sslParameters;
    private List<String> ciphers;
    private List<String> protocols;

    private SSLMaterial() {}

    public SSLContext getSslContext() {
        return sslContext;
    }

    public X509ExtendedKeyManager getKeyManager() {
        return keyManager;
    }

    public X509ExtendedTrustManager getTrustManager() {
        return trustManager;
    }

    public SSLParameters getSslParameters() {
        return sslParameters;
    }

    public HostnameVerifier getHostnameVerifier() {
        return hostnameVerifier;
    }

    public List<String> getCiphers() {
        return ciphers;
    }

    public List<String> getProtocols() {
        return protocols;
    }

    public static class Builder {

        private SSLContext sslContext;
        private X509ExtendedKeyManager keyManager;
        private X509ExtendedTrustManager trustManager;
        private HostnameVerifier hostnameVerifier;
        private SSLParameters sslParameters;
        private List<String> ciphers;
        private List<String> protocols;

        public Builder withSslContext(SSLContext sslContext) {
            this.sslContext = sslContext;
            return this;
        }

        public Builder withHostnameVerifier(HostnameVerifier hostnameVerifier) {
            this.hostnameVerifier = hostnameVerifier;
            return this;
        }

        public Builder withSslParameters(SSLParameters sslParameters) {
            this.sslParameters = sslParameters;
            return this;
        }

        public Builder withCiphers(List<String> ciphers) {
            this.ciphers = ciphers;
            return this;
        }

        public Builder withProtocols(List<String> protocols) {
            this.protocols = protocols;
            return this;
        }

        public Builder withKeyManager(X509ExtendedKeyManager keyManager) {
            this.keyManager = keyManager;
            return this;
        }

        public Builder withTrustManager(X509ExtendedTrustManager trustManager) {
            this.trustManager = trustManager;
            return this;
        }

        public SSLMaterial build() {
            SSLMaterial sslMaterial = new SSLMaterial();
            sslMaterial.sslContext = sslContext;
            sslMaterial.keyManager = keyManager;
            sslMaterial.trustManager = trustManager;
            sslMaterial.hostnameVerifier = hostnameVerifier;
            sslMaterial.sslParameters = sslParameters;
            sslMaterial.ciphers = ciphers;
            sslMaterial.protocols = protocols;
            return sslMaterial;
        }
    }

}
